@@ -1,9 +1,29 @@
 #' Create column specification
 #'
-#' @param ... Either column objects created by `col_*()`, or their
-#'   abbreviated character names. If you're only overriding a few columns,
-#'   it's best to refer to columns by name. If not named, the column types
-#'   must match the column names exactly.
+#' `cols()` includes all columns in the input data, guessing the column types
+#' as the default. `cols_only()` includes only the columns you explicitly
+#' specify, skipping the rest.
+#'
+#' The available specifications are: (with string abbreviations in brackets)
+#'
+#' * `col_logical()` \[l\], containing only `T`, `F`, `TRUE` or `FALSE`.
+#' * `col_integer()` \[i\], integers.
+#' * `col_double()` \[d\], doubles.
+#' * `col_character()` \[c\], everything else.
+#' * `col_factor(levels, ordered)` \[f\], a fixed set of values.
+#' * `col_date(format = "")` \[D\]: with the locale's `date_format`.
+#' * `col_time(format = "")` \[t\]: with the locale's `time_format`.
+#' * `col_datetime(format = "")` \[T\]: ISO8601 date times
+#' * `col_number()` \[n\], numbers containing the `grouping_mark`
+#' * `col_skip()` \[_, -\], don't import this column.
+#' * `col_guess()` \[?\], parse using the "best" type based on the input.
+#'
+#' @family parsers
+#' @param ... Either column objects created by `col_*()`, or their abbreviated
+#'   character names (as described in the \code{col_types} argument of
+#'   \code{\link{read_delim}}). If you're only overriding a few columns, it's
+#'   best to refer to columns by name. If not named, the column types must match
+#'   the column names exactly.
 #' @param .default Any named columns not explicitly overridden in `...`
 #'   will be read with this column type.
 #' @export
@@ -11,9 +31,24 @@
 #' cols(a = col_integer())
 #' cols_only(a = col_integer())
 #'
-#' # You can also use the standard abreviations
+#' # You can also use the standard abbreviations
 #' cols(a = "i")
 #' cols(a = "i", b = "d", c = "_")
+#'
+#' # You can also use multiple sets of column definitions by combining
+#' # them like so:
+#'
+#' t1 <- cols(
+#'   column_one = col_integer(),
+#'   column_two = col_number())
+#'
+#' t2 <- cols(
+#'  column_three = col_character())
+#'
+#' t3 <- t1
+#' t3$cols <- c(t1$cols, t2$cols)
+#' t3
+
 cols <- function(..., .default = col_guess()) {
   col_types <- list(...)
   is_character <- vapply(col_types, is.character, logical(1))
@@ -56,8 +91,16 @@ col_spec <- function(col_types, default = col_guess()) {
 
 is.col_spec <- function(x) inherits(x, "col_spec")
 
-as.col_spec <- function(x) UseMethod("as.col_spec")
 
+#' Generate a column specification
+#'
+#' This is most useful for generating a specification using the short form
+#' @param x Input object
+#' @keywords internal
+#' @examples
+#' as.col_spec("cccnnn")
+#' @export
+as.col_spec <- function(x) UseMethod("as.col_spec")
 #' @export
 as.col_spec.character <- function(x) {
   letters <- strsplit(x, "")[[1]]
@@ -179,12 +222,28 @@ show_cols_spec <- function(spec, n = getOption("readr.num_columns", 20)) {
   }
 }
 
+# This allows str() on a tibble object to print a little nicer.
+#' @export
+str.col_spec <- function(object, ..., indent.str = "") {
+
+  # Split the formatted column spec into strings
+  specs <- strsplit(format(object), "\n")[[1]]
+  cat(sep = "",
+    "\n",
+
+    # Append the current indentation string to the specs
+    paste(indent.str, specs, collapse = "\n"),
+
+    "\n")
+}
+
 
 #' Examine the column specifications for a data frame
 #'
 #' `spec()` extracts the full column specification from a tibble
 #' created by readr.
 #'
+#' @family parsers
 #' @param x The data frame object to extract from
 #' @return A col_spec object.
 #' @export
@@ -207,6 +266,7 @@ col_concise <- function(x) {
     c = col_character(),
     D = col_date(),
     d = col_double(),
+    f = col_factor(),
     i = col_integer(),
     l = col_logical(),
     n = col_number(),
