@@ -3,16 +3,16 @@ context("read_csv")
 test_that("read_csv col imputation, col_name detection and NA detection works", {
   test_data <- read_csv("basic-df.csv", col_types = NULL, col_names = TRUE, progress = FALSE)
   expect_equal(unname(unlist(lapply(test_data, class))),
-    c("logical", "integer", "numeric", "character"))
+    c("logical", "numeric", "numeric", "character"))
   expect_equal(names(test_data), c("a", "b", "c", "d"))
   expect_equal(sum(is.na(test_data$d)), 1)
 
-  test_data2 <- read_csv("basic-df.csv", col_types = list(a = "l", b = "i", c = "d", d = "c"), col_names = TRUE, progress = FALSE)
+  test_data2 <- read_csv("basic-df.csv", col_types = list(a = "l", b = "d", c = "d", d = "c"), col_names = TRUE, progress = FALSE)
   expect_identical(test_data, test_data2)
 })
 
 test_that("read_csv's 'NA' option genuinely changes the NA values", {
-  expect_equal(read_csv("a\nz", na = "z", progress = FALSE)$a, NA_character_)
+  expect_equal(read_csv("a\nz", na = "z", progress = FALSE)$a, NA)
 })
 
 test_that("read_csv's 'NA' option works with multiple NA values", {
@@ -28,6 +28,12 @@ test_that("passing \"\" to read_csv's 'NA' option reads \"\" correctly", {
   expect_equal(read_csv("a,b\nfoo,bar\nfoo,\n", na = "", progress = FALSE)$b, c("bar", NA))
 })
 
+test_that("changing read_csv's 'quote' argument works correctly", {
+  test_data <- read_csv("basic-df.csv", col_types = NULL, col_names = TRUE, progress = FALSE)
+  test_data_singlequote <- read_csv("basic-df-singlequote.csv", quote="'")
+  expect_identical(test_data, test_data_singlequote)
+})
+
 test_that("read_csv's 'skip' option allows for skipping'", {
   test_data <- read_csv("basic-df.csv", skip = 1, progress = FALSE)
   expect_equal(nrow(test_data), 9)
@@ -72,6 +78,16 @@ test_that("nuls are dropped with a warning", {
   expect_equal(x$abc, "ab")
 })
 
+test_that("can read from the clipboard", {
+  skip_if_no_clipboard()
+  clipr::write_clip("a,b,c\n1,2,3")
+  expect_identical(read_csv(clipboard()), read_csv("a,b,c\n1,2,3"))
+})
+
+test_that("can read from a multi-line character vector", {
+  expect_identical(nrow(read_csv(c("a,b,c", "1,2,3"))), 1L)
+})
+
 # Column warnings ---------------------------------------------------------
 
 test_that("warnings based on number of columns (not output columns)", {
@@ -138,7 +154,9 @@ test_that("too few or extra col_types generates warnings", {
 # read_csv2 ---------------------------------------------------------------
 
 test_that("decimal mark automatically set to ,", {
-  x <- read_csv2("x\n1,23", progress = FALSE)
+  expect_message(
+    x <- read_csv2("x\n1,23", progress = FALSE),
+    if (default_locale()$decimal_mark == ".") "decimal .*grouping .*mark" else NA)
   expect_equal(x[[1]], 1.23)
 })
 
@@ -189,9 +207,9 @@ test_that("comments are ignored regardless of where they appear", {
     x2 = c("B2", NA_character_, "A5"),
     x3 = c("C2", NA_character_, "A6"))
 
-  expect_equal(chk, out5)
-  expect_equal(chk, out6)
-  expect_equal(chk, out7)
+  expect_true(all.equal(chk, out5))
+  expect_true(all.equal(chk, out6))
+  expect_true(all.equal(chk, out7))
 })
 
 test_that("escaped/quoted comments are ignored", {
@@ -222,5 +240,38 @@ test_that("skip respects comments", {
 })
 
 test_that("read_csv returns an empty data.frame on an empty file", {
-   expect_equal(read_csv("empty-file", progress = FALSE), tibble::data_frame())
+   expect_true(all.equal(read_csv("empty-file", progress = FALSE), tibble::data_frame()))
+})
+
+test_that("read_delim errors on length 0 delimiter (557)", {
+  expect_error(read_delim("a b\n1 2\n", delim = ""),
+    "`delim` must be at least one character, use `read_table\\(\\)` for whitespace delimited input\\.")
+})
+
+test_that("read_csv does not duplicate header rows for leading whitespace (747)", {
+  x <- read_csv("\nfoo,bar\n1,2")
+  expect_equal(nrow(x), 1)
+  expect_equal(x$foo, 1)
+})
+
+test_that("read_csv handles whitespace between delimiters and quoted fields (668)", {
+  x <- read_csv('x,y\n1, \"hi,there\"\n3,4')
+  expect_equal(x$y, c("hi,there", "4"))
+})
+
+test_that("read_csv does not throw warnings for skipped columns (750, 833)", {
+  expect_warning(x <- read_csv("x,y\n1,2", col_types = "i_"), NA)
+})
+
+test_that("read_csv reads headers with embedded newlines (#784)", {
+  x <- read_csv("\"Header\nLine Two\"\nValue\n")
+  expect_equal(names(x), "Header\nLine Two")
+  expect_equal(x[[1]], "Value")
+})
+
+test_that("read_csv reads headers with embedded newlines 2 (#772)", {
+  x <- read_csv("\"Header\nLine Two\"\n\"Another line\nto\nskip\"\nValue,Value2\n", skip = 2, col_names = FALSE)
+  expect_equal(names(x), c("X1", "X2"))
+  expect_equal(x$X1, "Value")
+  expect_equal(x$X2, "Value2")
 })
